<?php

namespace Drupal\commerce_ecommpay\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_payment\Exception\DeclineException;
use Drupal\commerce_payment\Exception\InvalidResponseException;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use ecommpay\Callback;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\commerce_price\MinorUnitsConverterInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\commerce_ecommpay\EcommpayClient;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsRefundsInterface;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_price\Price;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsAuthorizationsInterface;

/**
 * Provides the Off-site Redirect payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "ecommpay",
 *   label = "ECOMMPAY",
 *   display_label = "ECOMMPAY",
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_ecommpay\PluginForm\EcommpayForm",
 *   },
 *   requires_billing_information = FALSE,
 * )
 */
class Ecommpay extends OffsitePaymentGatewayBase implements SupportsRefundsInterface, SupportsAuthorizationsInterface {

  /**
   * The ECOMMPAY client.
   *
   * @var Drupal\commerce_ecommpay\EcommpayClient
   */
  protected $client;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'project_id' => '',
      'secret_key' => '',
      'payment_methods' => [
        'card',
        'applepay',
        'googlepay',
        'klarna',
        'paypal',
        'open_banking',
      ],
    ] + parent::defaultConfiguration();
  }

  /**
   * Constructs a new PaymentGatewayBase object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\commerce_payment\PaymentTypeManager $payment_type_manager
   *   The payment type manager.
   * @param \Drupal\commerce_payment\PaymentMethodTypeManager $payment_method_type_manager
   *   The payment method type manager.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time.
   * @param \Drupal\commerce_price\MinorUnitsConverterInterface $minor_units_converter
   *   The minor units converter.
   * @param \Drupal\commerce_ecommpay\EcommpayClient $client
   *   The ECOMMPAY client.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    PaymentTypeManager $payment_type_manager,
    PaymentMethodTypeManager $payment_method_type_manager,
    TimeInterface $time,
    MinorUnitsConverterInterface $minor_units_converter = NULL,
    EcommpayClient $client) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $entity_type_manager,
      $payment_type_manager,
      $payment_method_type_manager,
      $time,
      $minor_units_converter);

    $this->client = $client;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('datetime.time'),
      $container->get('commerce_price.minor_units_converter'),
      $container->get('ecommpay.ecommpay_client')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['project_id'] = [
      '#type' => 'number',
      '#title' => $this->t('Project ID'),
      '#required' => TRUE,
      '#default_value' => $this->configuration['project_id'],
    ];

    $form['secret_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Secret Key'),
      '#required' => TRUE,
      '#default_value' => $this->configuration['secret_key'],
    ];

    $form['payment_methods'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Enabled payment methods'),
      '#required' => TRUE,
      '#options' => [
        'card' => 'Card',
        'applepay' => 'Apple Pay',
        'googlepay' => 'Google Pay',
        'klarna' => 'Klarna',
        'paypal' => 'PayPal',
        'open_banking' => 'Open Banking',
      ],
      '#default_value' => $this->configuration['payment_methods'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['project_id'] = $values['project_id'];
      $this->configuration['secret_key'] = $values['secret_key'];
      $this->configuration['payment_methods'] = $values['payment_methods'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $config = $this->getConfiguration();

    // Get payment_id from request.
    $payment_id = $request->query->get('payment_id');
    if (empty($payment_id)) {
      \Drupal::logger('commerce_ecommpay')->error('Missing payment_id');
      throw new InvalidResponseException('Missing payment_id');
    }

    // Get payment object.
    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
    $payments = $payment_storage->loadByProperties([
      'remote_id' => $payment_id,
    ]);
    if (empty($payments)) {
      \Drupal::logger('commerce_ecommpay')->error('No payments matching returned payment ID.');
      throw new InvalidResponseException('No payments matching returned payment ID.');
    }

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = reset($payments);
    if (!in_array($payment->getState(), ['authorization', 'completed'])) {
      // If local payment state is not completed,
      // please check the state using Gate API.
      $state = $this->client->getPaymentState(intval($config['project_id']), $config['secret_key'], $payment_id);

      if (!in_array($state, [Callback::AW_CAP_STATUS, Callback::SUCCESS_STATUS])) {
        \Drupal::logger('commerce_ecommpay')->info('Payment was not successful.');
        throw new DeclineException('The was a problem processing you payment. Please try again.');
      }

      // Set correct state on Drupal payment entity.
      switch ($state) {
        case Callback::AW_CAP_STATUS:
          $payment->setState('authorization');
          break;

        case Callback::SUCCESS_STATUS:
          $payment->setState('completed');
          break;
      }
      $payment->save();

    }
  }

  /**
   * {@inheritdoc}
   */
  public function onNotify(Request $request) {
    $config = $this->getConfiguration();

    // Check if secret_key config is present.
    if (empty($config['secret_key'])) {
      \Drupal::logger('commerce_ecommpay')->error('The "Secret Key" configuration value must be set.');
      throw new PaymentGatewayException('The "Secret Key" configuration value must be set.');
    }

    try {
      $callback = $this->client->parseCallbackBody($config['secret_key'], $request->getContent());
    }
    catch (\Exception $e) {
      \Drupal::logger('commerce_ecommpay')->error($e->getMessage());
      throw new InvalidResponseException($e->getMessage());
    }

    // Get payment Id.
    $payment_id = $callback->getPaymentId();

    // Load Drupal payment object.
    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
    $payments = $payment_storage->loadByProperties([
      'remote_id' => $payment_id,
    ]);

    if (empty($payments)) {
      \Drupal::logger('commerce_ecommpay')->error('No payments matching returned payment ID.');
      throw new InvalidResponseException('No payments matching returned payment ID.');
    }

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = reset($payments);
    if (empty($payment)) {
      \Drupal::logger('commerce_ecommpay')->error('No payments matching returned payment ID.');
      throw new InvalidResponseException('No payments matching returned payment ID.');
    }

    // Handle statuses accordingly.
    switch ($callback->getPaymentStatus()) {
      // Success.
      case Callback::SUCCESS_STATUS:
        $payment->setState('completed');
        break;

      // Awaiting Capture.
      case Callback::AW_CAP_STATUS:
        $payment->setState('authorize');
        break;
    }

    $payment->setRemoteState($callback->getPaymentStatus());
    $payment->save();
  }

  /**
   * {@inheritdoc}
   */
  public function refundPayment(PaymentInterface $payment, Price $amount = NULL) {
    $config = $this->getConfiguration();

    // Refund payment.
    $state = $this->client->refundPayment(
      intval($config['project_id']),
      $config['secret_key'],
      $payment->getRemoteId(),
      $this->minorUnitsConverter->toMinorUnits($amount),
      $amount->getCurrencyCode());

    if ($state['status'] != 'success') {
      \Drupal::logger('commerce_ecommpay')->info('Payment was not refunded.');
      throw new PaymentGatewayException('The was a problem refunding your payment. Please try again.');
    }
    else {
      $old_refunded_amount = $payment->getRefundedAmount();
      $new_refunded_amount = $old_refunded_amount->add($amount);
      if ($new_refunded_amount->lessThan($payment->getAmount())) {
        $payment->setState('partially_refunded');
      }
      else {
        $payment->setState('refunded');
      }

      $payment->setRefundedAmount($new_refunded_amount);
      $payment->save();
    }

  }

  /**
   * {@inheritdoc}
   */
  public function capturePayment(PaymentInterface $payment, Price $amount = NULL) {
    $config = $this->getConfiguration();

    // Capture payment.
    $state = $this->client->capturePayment(
      intval($config['project_id']),
      $config['secret_key'],
      $payment->getRemoteId(),
      $this->minorUnitsConverter->toMinorUnits($amount),
      $amount->getCurrencyCode());

    if ($state['status'] != 'success') {
      \Drupal::logger('commerce_ecommpay')->info('Payment was not captured.');
      throw new PaymentGatewayException('The was a problem capturing your payment. Please try again.');
    }
    else {
      $payment->setState('completed');
      $payment->save();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function voidPayment(PaymentInterface $payment) {
    $config = $this->getConfiguration();

    // Capture payment.
    $state = $this->client->voidPayment(
      intval($config['project_id']),
      $config['secret_key'],
      $payment->getRemoteId());

    if ($state['status'] != 'success') {
      \Drupal::logger('commerce_ecommpay')->info('Payment was not voided.');
      throw new PaymentGatewayException('The was a problem voiding your payment. Please try again.');
    }
    else {
      $payment->setState('voided');
      $payment->save();
    }

  }

}
