<?php

namespace Drupal\commerce_ecommpay\PluginForm;

use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use ecommpay\exception\ValidationException;
use Drupal\user\Entity\User;
use Drupal\commerce_ecommpay\EcommpayClient;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\commerce_price\MinorUnitsConverterInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Component\Uuid\Php;

/**
 * Class EcommpayForm.
 */
class EcommpayForm extends BasePaymentOffsiteForm implements ContainerInjectionInterface {
  /**
   * The ECOMMPAY client.
   *
   * @var \Drupal\commerce_ecommpay\EcommpayClient
   */
  protected $client;

  /**
   * Minor units converter.
   *
   * @var \Drupal\commerce_price\MinorUnitsConverterInterface
   * */
  protected $minorUnitsConverter;

  /**
   * Language Manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   * */
  protected $languageManager;

  /**
   * UUID generator.
   *
   * @var \Drupal\Component\Uuid\Uuid
   * */
  protected $uuid;

  /**
   * Constructs a new EcommpayForm object.
   *
   * @param \Drupal\commerce_ecommpay\EcommpayClient $client
   *   The ECOMMPAY client.
   * @param \Drupal\commerce_price\MinorUnitsConverterInterface $minor_units_converter
   *   The minor units converter.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Component\Uuid\Php $uuid
   *   The UUID generator.
   */
  public function __construct(EcommpayClient $client, MinorUnitsConverterInterface $minor_units_converter, LanguageManagerInterface $language_manager, Php $uuid) {
    $this->client = $client;
    $this->minorUnitsConverter = $minor_units_converter;
    $this->languageManager = $language_manager;
    $this->uuid = $uuid;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
      $container->get('ecommpay.ecommpay_client'),
      $container->get('commerce_price.minor_units_converter'),
      $container->get('language_manager'),
      $container->get('uuid'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;

    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $config = $payment_gateway_plugin->getConfiguration();

    if (empty($config['project_id'])) {
      throw new PaymentGatewayException('The "Project ID" configuration value must be set.');
    }

    if (empty($config['secret_key'])) {
      throw new PaymentGatewayException('The "Secret Key" configuration value must be set.');
    }

    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $payment->getOrder();

    /** @var \Drupal\user\Entity\User $user */
    $user = \Drupal::currentUser();

    // Get current language.
    $language_code = $this->languageManager->getCurrentLanguage()->getId();

    // Determine customer_id. If user is logged in, use user's assigned uuid,
    // otherwise generate unique UUID.
    $customer_id = $this->uuid->generate();
    if (!$user->isAnonymous()) {
      /** @var \Drupal\user\Entity\User $user */
      $user = User::load($user->id());
      $customer_id = $user->uuid();
    }

    // Get customer's email.
    $email = $order->getEmail();

    // Get billing address.
    /** @var \Drupal\address\AddressInterface $billing_address */
    $billing_address = NULL;
    if ($order->getBillingProfile()) {
      $billing_address = $order->getBillingProfile()->get('address')->first();
    }

    // We generate our own UUID here to make sure this value is always unique.
    // ECOMMPAY requires this value to be unique per project.
    // We also allow other modules to alter this value if need be.
    $payment_id = $this->uuid->generate();
    \Drupal::moduleHandler()->alter('commerce_ecommpay_payment_id', $remote_payment_id, $payment);

    try {
      $url = $this->client->getPaymentPageUrl(
        $config['project_id'],
        $config['secret_key'],
        $payment_id,
        $customer_id,
        $email,
        $billing_address,
        $this->minorUnitsConverter->toMinorUnits($order->getTotalPrice()),
        $order->getTotalPrice()->getCurrencyCode(),
        $form['#return_url'],
        $form['#return_url'],
        $payment_gateway_plugin->getNotifyUrl()->toString(),
        $language_code,
        $form['#capture'] ? 'sale' : 'auth');
    }
    catch (ValidationException $e) { // Validating possible exceptions
      \Drupal::logger('ecommpay')->error($e->getFormattedMessage());
      throw new PaymentGatewayException($e->getFormattedMessage());
    }

    // Set remote_payment_id for payment and save payment object.
    $payment->setRemoteId($payment_id);
    $payment->save();

    // Manage presentation, redirect, modal or iframe.
    $form['container'] = [
      '#type' => 'html_tag',
      '#tag' => 'iframe',
      '#attributes' => [
        'src' => $url,
        'width' => "100%",
        'height' => "600px",
        'loading' => 'eager',
        'class' => ['ecommpay-payment-iframe']
      ],
    ];

    // Attach url to drupal settings.
    $form['#attached']['drupalSettings']['ecommpay'] = [
      'url' => $url
    ];

    return $this->buildRedirectForm($form, $form_state, $url, []);
  }

}
